/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.ruleengine.expression;

import org.apache.commons.beanutils.PropertyUtils;
import org.tinygroup.commons.match.SimpleTypeMatcher;
import org.tinygroup.format.Formater;
import org.tinygroup.format.impl.ContextFormater;
import org.tinygroup.format.impl.FormaterImpl;
import org.tinygroup.ruleengine.RuleSession;
import org.tinygroup.ruleengine.exception.PropertyGetException;

public abstract class Expression<T> implements IExpression<T> {
	private transient RuleSession ruleSession;
	private static transient Formater formater = new FormaterImpl();
	static {
		formater.addFormatProvider("", new ContextFormater());
		formater.addFormatProvider("in", new ContextFormater());
	}

	public static Formater getFormater() {
		return formater;
	}

	public RuleSession getRuleSession() {
		return ruleSession;
	}

	public void setRuleSession(RuleSession ruleSession) {
		this.ruleSession = ruleSession;
	}

	private Object getObject(String strArgs) {
		String str = strArgs;
		if (str instanceof String) {
			str = formater.format(ruleSession.getContext(), str);
		}

		// 所有的都不是，说明是对象或表达式,此时返回null
		Object o = null;
		if (str != null) {
			str = str.trim();
			o = SimpleTypeMatcher.matchType(str);
		}
		return o;
	}

	/**
	 * 根据名称获取对象，先从本地环境取数据，接下来从工作内存取数据
	 * 
	 * @param name
	 * @return
	 */
	protected Object getObjectByName(String name) {
		Object object = null;
		int index = name.indexOf('.');
		if (index == -1) {
			object = ruleSession.getContext().get(name);
		} else {
			String k = name.substring(0, index);
			String p = name.substring(index + 1);
			object = ruleSession.getContext().get(k);
			if (object != null) {
				try {
					object = PropertyUtils.getProperty(object, p);
				} catch (Exception e) {
					throw new PropertyGetException(e);
				}
			}
		}
		if (object == null) {
			object = getObject(name);
		}
		return object;
	}

	protected Object getProperty(Object object, String propertyName) {
		try {
			return PropertyUtils.getProperty(object, propertyName);
		} catch (Exception e) {
			throw new PropertyGetException(e);
		}
	}

}
