/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.ruleengine.expression.common;

import java.lang.reflect.Array;
import java.util.Collection;

import org.tinygroup.ruleengine.expression.ExpressionContainer;
import org.tinygroup.ruleengine.expression.IExpression;

/**
 * @author luoguo
 */
@SuppressWarnings("rawtypes")
public class ForEachClause extends ExpressionContainer {
	private String collection;
	private String varName;

	public String getCollection() {
		return collection;
	}

	public void setCollection(String collection) {
		this.collection = collection;
	}

	public String getVarName() {
		return varName;
	}

	public void setVarName(String varName) {
		this.varName = varName;
	}

	public String getName() {
		return "foreach";
	}

	public Object evaluate() {
		Object ot = getObjectByName(collection);
		if (ot instanceof Collection) {
			Collection objectCollection = (Collection) ot;
			for (Object o : objectCollection) {
				getRuleSession().getContext().put(varName, o);
				if (getSubExpressions() != null) {
					for (Object e : getSubExpressions()) {
						IExpression expression = (IExpression) e;
						expression.setRuleSession(getRuleSession());
						expression.evaluate();
					}
				}
			}
		} else if (ot.getClass().isArray()) {
			for (int i = 0; i < Array.getLength(ot); i++) {
				Object o = Array.get(ot, i);
				getRuleSession().getContext().put(varName, o);
				if (getSubExpressions() != null) {
					for (Object e : getSubExpressions()) {
						IExpression expression = (IExpression) e;
						expression.setRuleSession(getRuleSession());
						expression.evaluate();
					}
				}
			}
		}
		return null;
	}
}
