/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.ruleengine.impl;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.List;

import org.tinygroup.context.Context;
import org.tinygroup.context.impl.ContextImpl;
import org.tinygroup.ruleengine.RuleSession;
import org.tinygroup.ruleengine.Rule;
import org.tinygroup.ruleengine.RuleSet;
import org.tinygroup.ruleengine.RuleSetExecuteException;

/**
 * 注意：如果变量名冲突，会导致前面的会被后面的覆盖，目前没有防冲突
 * 防冲突会导致比较大的性能消耗
 * @author luoguo
 *
 */
public class RuleSessionImpl implements RuleSession {

	private RuleSet ruleSet;
	private Context context = new ContextImpl();

	public Map<String, Object> execute(RuleSet ruleSet,
			Context context) throws RuleSetExecuteException {
		Map<String, Object> result = new HashMap<String, Object>();
		this.ruleSet = ruleSet;
		context.putSubContext(this.context.hashCode() + "", this.context);
		if (ruleSet == null) {
			throw new RuleSetExecuteException("规则集不能为空");
		}
		if (!checkInput()) {
			throw new RuleSetExecuteException("输入参数不足。");
		}
		List<Rule> rules = agandaRule(ruleSet);

		if (ruleSet.getRules() != null) {
			for (Rule rule : rules) {
				boolean executeAction = checkCondition(rule);
				if (executeAction) {
					if (rule.getActionExpression() != null) {
						rule.getActionExpression().setRuleSession(this);
						rule.getActionExpression().evaluate();
					}
				} else {
					if (rule.getElseActionExpression() != null) {
						rule.getElseActionExpression().setRuleSession(this);
						rule.getElseActionExpression().evaluate();
					}
				}
			}
		}
		if (!checkOutput(result)) {
			throw new RuleSetExecuteException("输出参数不足。");
		}
		return result;
	}

	private boolean checkCondition(Rule rule) {
		boolean executeAction = true;
		if (rule.getBoolExpression() != null) {
			rule.getBoolExpression().setRuleSession(this);
			if (rule.getBoolExpression().evaluate().equals(Boolean.FALSE)) {
				executeAction = false;
			}
		}
		return executeAction;
	}

	public boolean checkInput() {
		/*
		List<Parameter> para = ruleSet.getParameters();
		if (para != null) {
			for (Parameter p : para) {
				if (!p.getDirect().equals(Parameter.DIRECT_OUTPUT)) {
					Object object = context.get(p.getName());
					if (p.isRequired() && object == null) {
						return false;
					}
					if (ruleSet.getClassAliases() == null) {
						if (!ClassAliases.getFullClassName(p.getType()).equals(
								ClassAliases.getFullClassName(object.getClass()
										.getName()))) {
							return false;
						}
					} else {
						if (!ruleSet.getClassAliases().compareClassName(
								object.getClass().getName(), p.getType())) {
							return false;
						}
					}
				}
			}
		}
		*/
		return true;
	}

	// TODO 类型如果是父类，现在不能判断出来，因此，如果返回的类型是设定类型的子类也是可以的
	public boolean checkOutput(Map<String, Object> result) {
		/*
		List<Parameter> para = ruleSet.getParameters();
		if (para != null) {
			for (Parameter p : para) {
				if (!p.getDirect().equals(Parameter.DIRECT_INPUT)) {
					Object object = context.get(p.getName());
					if (p.isRequired() && object == null) {
						return false;
					}
					if (object != null) {
						if (ruleSet.getClassAliases() == null) {
							if (!ClassAliases.getFullClassName(p.getType())
									.equals(ClassAliases
											.getFullClassName(object.getClass()
													.getName()))) {
								return false;
							} else {
								result.put(p.getName(), object);
							}
						} else {
							if (!ruleSet.getClassAliases().compareClassName(
									object.getClass().getName(), p.getType())) {
								return false;
							} else {
								result.put(p.getName(), object);
							}
						}
					}
				}
			}
		}*/
		return true;
	}

	public RuleSet getRuleSet() {
		return ruleSet;
	}

	public Context getContext() {
		return context;
	}

	public List<Rule> agandaRule(RuleSet ruleSet) {
		Collections.sort(ruleSet.getRules());
		return ruleSet.getRules();
	}

}
